@@ -17,7 +17,7 @@
 
 */
 
-//#define DEBUG 1
+// #define DEBUG 1
 
 #include <modglue/pipe.hh>
 #include <modglue/process.hh>
@@ -211,6 +211,18 @@ ActionAddCell::ActionAddCell(Glib::RefPtr<DataCell> toadd, Glib::RefPtr<DataCell
 void ActionAddCell::execute(XCadabra& xc)
 	{
 	xc.add_cell(cell, ref, before);
+	xc.show_cell(cell);
+	xc.active_canvas->cell_grab_focus(cell);
+
+	// Then insert the associated cells, if there are any
+	if(associated_cells.size()>0) {
+		Glib::RefPtr<DataCell> previous_cell=cell;
+		for(size_t i=0; i<associated_cells.size(); ++i) {
+			xc.add_cell(associated_cells[i], previous_cell, false); 
+			xc.show_cell(associated_cells[i]);
+			previous_cell=associated_cells[i];
+			}
+		}
 	}
 
 void ActionAddCell::revert(XCadabra& xc)
@@ -223,10 +235,14 @@ void ActionAddCell::revert(XCadabra& xc)
 	--prevcell;
 
 	// Reverting a cell add operation should also remove all cells which got
-	// generated from it. We don't keep their info though (contrast this with remove cell).
+	// generated from it. We store a list of these associated cells, in case we
+	// get re-done again.
+	associated_cells.clear();
 	while(fnd!=xc.datacells.end()) {
 		for(unsigned int i=0; i<xc.canvasses.size(); ++i)  
 			xc.canvasses[i]->remove_cell(*fnd);
+		if(*fnd != cell)
+			associated_cells.push_back(*fnd);
 		fnd=xc.datacells.erase(fnd);
 		if(fnd==xc.datacells.end() || (*fnd)->cell_type==DataCell::c_input || (*fnd)->cell_type==DataCell::c_tex)
 			 break;
@@ -235,8 +251,7 @@ void ActionAddCell::revert(XCadabra& xc)
 	// We have to put the cursor in the previous input cell or in the previous open TeX cell.
 	// So walk up the cells until we meet this condition.
 	while((*prevcell)->cell_type!=DataCell::c_input 
-			&& !( ( (*prevcell)->cell_type==DataCell::c_comment || (*prevcell)->cell_type==DataCell::c_texcomment ) 
-					&& (*prevcell)->tex_hidden==false) )
+			&& !( (*prevcell)->cell_type==DataCell::c_tex && (*prevcell)->tex_hidden==false) )
 		--prevcell;
 	
 	xc.active_canvas->cell_grab_focus(*prevcell);
@@ -257,6 +272,12 @@ void ActionRemoveCell::execute(XCadabra& xc)
 	xc.selected=0;
 	XCadabra::DataCells_t::iterator fnd=std::find(xc.datacells.begin(), xc.datacells.end(), cell);
 	assert(fnd!=xc.datacells.end());
+	assert(fnd!=xc.datacells.begin());
+
+	// Store a pointer to the previous DataCell.
+	XCadabra::DataCells_t::iterator fnd_prev=fnd;
+	--fnd_prev;
+	prev_cell=(*fnd_prev);
 
 	// Remove cells until an input or tex cell is found.
 	associated_cells.clear();
@@ -272,39 +293,52 @@ void ActionRemoveCell::execute(XCadabra& xc)
 
 	// Position the cursor.
 	if(fnd!=xc.datacells.end()) {
-		next_cell=(*fnd);
-
 		// We have to put the cursor in the next input cell or in the next open TeX cell.
 		// So walk down the cells until we meet this condition (or end).
 		while(fnd!=xc.datacells.end() 
 				&& (*fnd)->cell_type!=DataCell::c_input 
-				&& !( ( (*fnd)->cell_type==DataCell::c_comment || (*fnd)->cell_type==DataCell::c_texcomment) 
-						&& (*fnd)->tex_hidden==false) )
+				&& !( (*fnd)->cell_type==DataCell::c_tex && (*fnd)->tex_hidden==false) )
 			 ++fnd;
 
 		if(fnd!=xc.datacells.end())
 			 xc.active_canvas->cell_grab_focus(*fnd);
 		}
 	else {
-		// There is no cell below; create a new one and put the cursor there.
-		Glib::RefPtr<DataCell> newcell(new DataCell(DataCell::c_input));
-		next_cell=newcell;
-		xc.add_cell(newcell, Glib::RefPtr<DataCell>() );
-		xc.active_canvas->cell_grab_focus(newcell);
+		// There is no cell below (i.e. we just deleted the last cell). 
+		// Find the last input cell of the notebook and activate that one.
+		// (i.e. we walk upward here, as opposed to the normal action, which is to 
+		// move one input cell down).
+		--fnd;
+		while(fnd!=xc.datacells.begin() 
+				&& (*fnd)->cell_type!=DataCell::c_input 
+				&& !( (*fnd)->cell_type==DataCell::c_tex && (*fnd)->tex_hidden==false) ) {
+			--fnd;
+			}
+		xc.active_canvas->cell_grab_focus(*fnd);
 		}
 	}
 
 void ActionRemoveCell::revert(XCadabra& xc)
 	{
-	// Find next_cell in the currently active cells.
-	XCadabra::DataCells_t::iterator fnd=std::find(xc.datacells.begin(), xc.datacells.end(), next_cell);
+	// Find prev_cell in the currently active cells.
+	XCadabra::DataCells_t::iterator fnd=std::find(xc.datacells.begin(), xc.datacells.end(), prev_cell);
 	assert(fnd!=xc.datacells.end());
 
-	// Re-insert all these cells back into the tree.
-	xc.add_cell(cell, (*fnd) );
-//	std::cerr << associated_cells.size() << " cells to add" << std::endl;
-	for(size_t i=0; i<associated_cells.size(); ++i)
-		xc.add_cell(associated_cells[i], (*fnd));
+	// Re-insert the input cell back into the tree.
+	xc.add_cell(cell, (*fnd), false);
+	xc.show_cell(cell);
+
+	// Then insert the associated cells.
+	if(associated_cells.size()>0) {
+		Glib::RefPtr<DataCell> previous_cell=cell;
+		for(size_t i=0; i<associated_cells.size(); ++i) {
+			xc.add_cell(associated_cells[i], previous_cell, false); 
+			xc.show_cell(associated_cells[i]);
+			previous_cell=associated_cells[i];
+			}
+		}
+
+	xc.active_canvas->cell_grab_focus(cell);
 	}
 
 DataCell::DataCell(cell_t ct, const std::string& str, bool texhidden)
@@ -459,8 +493,8 @@ VisualCell *NotebookCanvas::add_cell(Glib::RefPtr<DataCell> dc, Glib::RefPtr<Dat
 			gtk_box_set_child_packing(((Gtk::Box *)(&scrollbox))->gobj(), 
 											  ((Gtk::Widget *)(newcell->inbox))->gobj(),
 											  false, false, 10, GTK_PACK_START);
-			while (gtk_events_pending ())
-				gtk_main_iteration ();
+//			while (gtk_events_pending ())
+//				gtk_main_iteration ();
 
 			// Connect to signal for 'ctrl-enter pressed', i.e. 'feed this cell to the kernel'.
 			newcell->inbox->edit.emitter.connect(
@@ -475,7 +509,7 @@ VisualCell *NotebookCanvas::add_cell(Glib::RefPtr<DataCell> dc, Glib::RefPtr<Dat
 			newcell->inbox->edit.content_changed.connect(
 				 sigc::bind<VisualCell *>(
 					 sigc::mem_fun(this, &NotebookCanvas::scroll_into_view_callback), newcell));
-	
+
 			break;
 			}
 		case DataCell::c_error:
@@ -658,13 +692,13 @@ bool NotebookCanvas::scroll_into_view(VisualCell *vc, bool center)
 		 va->set_value(aim);
 		 }
 	else {
-		 if(al.get_y() + rect.get_y() < upper_visible) 
-			  va->set_value(std::max(0.0, (double)(al.get_y() + rect.get_y() - LINE_SPACING)));
-		 else {
-			  if(al.get_y() + rect.get_y() + rect.get_height() > lower_visible) 
-					va->set_value(al.get_y() + rect.get_y() + rect.get_height() - va->get_page_size());
-			  }
-		 }
+		if(al.get_y() + rect.get_y() < upper_visible) // cell is above top of view
+			va->set_value(std::max(0.0, (double)(al.get_y() + rect.get_y() - LINE_SPACING)));
+		else {
+			if(al.get_y() + rect.get_y() + rect.get_height() > lower_visible) // cell is below bottom of view
+				va->set_value(al.get_y() + rect.get_y() + rect.get_height() - va->get_page_size() + 20);
+			}
+		}
 
 	return false;
 	}
@@ -727,7 +761,7 @@ void NotebookCanvas::show()
 
 
 XCadabra::XCadabra(modglue::ext_process& cdbproc, const std::string& filename, modglue::main *mm)
-	: font_step(0), disable_stacks(false), hglass(Gdk::WATCH),
+	: font_step(0), brain_wired(0), disable_stacks(false), hglass(Gdk::WATCH),
 	  load_file(false), have_received(false), cmm(mm), name(filename), modified(false), running(false),
 	  running_last(0), restarting_kernel(false),
 	  last_used_id(0),
@@ -735,7 +769,7 @@ XCadabra::XCadabra(modglue::ext_process& cdbproc, const std::string& filename, m
 	  b_kernelversion("Kernel version: not running"),
 	  b_help(Gtk::Stock::HELP), b_stop(Gtk::Stock::STOP), b_undo(Gtk::Stock::UNDO), b_redo(Gtk::Stock::REDO),
 	  last_configure_width(0),
-     cdb(cdbproc), selected(0)
+     cdb(cdbproc), selected(0), to_scroll_to(0)
 	{
 	std::string res=load_config();
 	if(res.size()>0) 
@@ -783,6 +817,7 @@ XCadabra::XCadabra(modglue::ext_process& cdbproc, const std::string& filename, m
 	actiongroup->add( Gtk::Action::create("MenuSettings", "_Settings") );
 	actiongroup->add( Gtk::Action::create("MenuTutorial", "_Tutorial") );
 	actiongroup->add( Gtk::Action::create("MenuFontSize", "Font size") );
+	actiongroup->add( Gtk::Action::create("MenuBrainWired", "Brain wired for") );
 	actiongroup->add( Gtk::Action::create("MenuHelp", "_Help") );
 
 	actiongroup->add( Gtk::Action::create("New", Gtk::Stock::NEW),
@@ -854,6 +889,22 @@ XCadabra::XCadabra(modglue::ext_process& cdbproc, const std::string& filename, m
 	actiongroup->add( font_action3, sigc::bind(sigc::mem_fun(*this, &XCadabra::on_settings_font_size), 4));
 	if(font_step==4) font_action3->set_active();
 
+	// Key wiring
+
+	Gtk::RadioAction::Group group_brain_wired;
+
+	brain_wired_action0=Gtk::RadioAction::create(group_brain_wired, "BrainEmacs", "Emacs keys");
+	brain_wired_action0->property_value() = 0;
+	actiongroup->add( brain_wired_action0, sigc::bind(sigc::mem_fun(*this, &XCadabra::on_settings_brain), 0 ));
+	if(brain_wired==0) brain_wired_action0->set_active();
+
+	brain_wired_action1=Gtk::RadioAction::create(group_brain_wired, "BrainWindoze", "Windoze keys");
+	brain_wired_action1->property_value() = 1;
+	actiongroup->add( brain_wired_action1, sigc::bind(sigc::mem_fun(*this, &XCadabra::on_settings_brain), 1 ));
+	if(brain_wired==1) brain_wired_action1->set_active();
+
+	// Construct menu
+
 	actiongroup->add( Gtk::Action::create("Basics", "Basics"),
 								  sigc::bind(sigc::mem_fun(*this, &XCadabra::on_tutorial_open),0) );   
 	actiongroup->add( Gtk::Action::create("Derivatives", "Derivatives"),
@@ -916,6 +967,10 @@ XCadabra::XCadabra(modglue::ext_process& cdbproc, const std::string& filename, m
 		"         <menuitem action='FontMedium'/>"
 		"         <menuitem action='FontLarge'/>"
 		"         <menuitem action='FontExtraLarge'/>"
+      "      </menu>"
+		"      <menu action='MenuBrainWired'>"
+		"         <menuitem action='BrainEmacs'/>"
+		"         <menuitem action='BrainWindoze'/>"
       "      </menu>"
 		"    </menu>"
 		"    <menu action='MenuHelp'>"
@@ -1004,11 +1059,17 @@ bool XCadabra::on_key_press_event(GdkEventKey* event)
 	{
 	switch(event->keyval) {
 		case GDK_Home:
-			active_canvas->scroll_to_start();
-			return true; // if we don't return immediately, selections will go away
+			if(brain_wired==0 || (event->state&Gdk::CONTROL_MASK) ) {
+				active_canvas->scroll_to_start();
+				return true; // if we don't return immediately, selections will go away
+				}
+			else break;
 		case GDK_End:
-			active_canvas->scroll_to_end();
-			return true; // if we don't return immediately, selections will go away
+			if(brain_wired==0 || (event->state&Gdk::CONTROL_MASK) ) {
+				active_canvas->scroll_to_end();
+				return true; // if we don't return immediately, selections will go away
+				}
+			else break;
 		case GDK_Page_Up:
 			active_canvas->scroll_up();
 			return true; // if we don't return immediately, selections will go away
@@ -1017,7 +1078,8 @@ bool XCadabra::on_key_press_event(GdkEventKey* event)
 			return true; // if we don't return immediately, selections will go away
 		case 108:
 			if( (event->state&Gdk::CONTROL_MASK) ) { // Ctrl-L: center display
-				active_canvas->scroll_into_view(active_cell, true);
+				set_next_into_view(active_cell);
+//				active_canvas->scroll_into_view(active_cell, true);
 				return true; // if we don't return immediately, selections will go away
 				}
 			break;
@@ -1044,7 +1106,8 @@ bool XCadabra::on_key_press_event(GdkEventKey* event)
 	// etc. After that we update the notebook position if required.
 	bool retval=Window::on_key_press_event(event);
 	if(active_cell && event->keyval < 65000) // FIXME: how to find dead keys?
-		 active_canvas->scroll_into_view(active_cell);
+		set_next_into_view(active_cell);
+		//active_canvas->scroll_into_view(active_cell);
 	
 	return retval;
 	}
@@ -1089,7 +1152,15 @@ void XCadabra::on_stop()
 void XCadabra::on_kill()
 	{
 	restarting_kernel=true;
-	*(cdb.output_pipe("stdin")) << "@quit;\n" << std::flush;
+
+   //	*(cdb.output_pipe("stdin")) << "@quit;\n" << std::flush;
+
+	// There is no point in shutting down cadabra nicely (as we did with
+	// the statement above in earlier versions); instead, we send a full
+	// blown SIGKILL.
+	if(kill(cdb.get_pid(),0)==0)
+		kill(cdb.get_pid(), SIGKILL);
+
 	b_cdbstatus.set_text(" Status: Restarting kernel...");
 	b_kernelversion.set_text("Kernel: not running");
 
@@ -1157,10 +1228,10 @@ void XCadabra::on_help_about()
 	{
 	Gtk::AboutDialog md;
 	md.set_name("XCadabra");
-	md.set_website("http://www.aei.mpg.de/~peekas/cadabra/");
+	md.set_website("http://cadabra.phi-sci.com/");
 	md.set_website_label("cadabra website");
 	md.set_version(RELEASE);
-	md.set_copyright("\xC2\xA9 2006-2009 Kasper Peeters");
+	md.set_copyright("\xC2\xA9 2006-2010 Kasper Peeters");
 	md.set_comments("Graphical user interface for the cadabra symbolic computer algebra system.");
 	md.set_license("XCadabra and Cadabra are available under the Gnu General Public License version 2.\n\nIf you use Cadabra or even just play with it, I would like to hear about it. Please send me an email so that I can get an idea of who is interested in this program.\n\nIf you use Cadabra in your own work, please cite both\n\nKasper Peeters\n\"A field-theory motivated approach to computer algebra\"\ncs.sc/0608005\nComput. Phys. Commun 176 (2007) 550\n\nKasper Peeters\n\"Introducing Cadabra: a symbolic computer algebra system for field theory problems\"\nhep-th/0701238\n\nThank you!");
 #if (GTKMM_VER == 28 || GTKMM_VER == 212 || GTKMM_VER == 216)
@@ -1401,8 +1472,22 @@ bool XCadabra::on_kernel_exit(modglue::ext_process& pr)
 	connect_io_signals();
 
 	*(cdb.output_pipe("stdin")) << "@print_status{true};\n" << std::flush;
+	kernel_idle();
 	b_cdbstatus.set_text(" Status: Kernel restarted, idle.");
+	
+	// Remove all cell-id to datacell pointer mappings, since any of those
+	// which we are still waiting for are now invalid.
+	id_to_datacell.clear();
 
+	// Remove the 'running' flag from all cells, since we have a new kernel
+	// now, and none of the cells are being processed anymore.
+	DataCells_t::iterator dit=datacells.begin();
+	while(dit!=datacells.end()) {
+		(*dit)->running=false;
+		++dit;
+		}
+	restarting_kernel=false;
+	
 
 	// return 'false' to keep the main loop running.
 	return false;
@@ -1592,6 +1677,9 @@ void XCadabra::show_cell(Glib::RefPtr<DataCell> datacell)
 
 bool XCadabra::handle_editbox_output(std::string str, NotebookCanvas *can, VisualCell *vis)
 	{
+#ifdef DEBUG
+	std::cerr << "handling edit box" << std::endl;
+#endif
 	// Disable this cell until output has been received, so we
 	// don't get it twice.
 	if(vis->datacell->running==false) {
@@ -1610,7 +1698,7 @@ bool XCadabra::handle_editbox_output(std::string str, NotebookCanvas *can, Visua
 											 << str << "\n"
 											 << "#cellend\n" << std::flush;
 #ifdef DEBUG
-		std::cerr << "sending done" << std::endl;
+		std::cerr << "sending of cell # " << last_used_id << " done" << std::endl;
 #endif
 		}
 	return true;
@@ -1669,10 +1757,11 @@ void XCadabra::handle_on_grab_focus(NotebookCanvas *can, VisualCell *vis)
 	if(running && vis) {
 		if(running_last==active_cell->datacell) {
 			kernel_idle();
-			while (gtk_events_pending ())
-				gtk_main_iteration ();
+//			while (gtk_events_pending ())
+//				gtk_main_iteration ();
 			// make sure this last cell is in view
-			active_canvas->scroll_into_view(active_cell);
+//			active_canvas->scroll_into_view(active_cell);
+			set_next_into_view(active_cell);
 			}
 		else {
 			Glib::RefPtr<Gtk::TextBuffer> textbuf=active_cell->datacell->textbuf;
@@ -1682,10 +1771,11 @@ void XCadabra::handle_on_grab_focus(NotebookCanvas *can, VisualCell *vis)
 				std::cerr << "cell does not end with delimiter" << std::endl;
 #endif
 				kernel_idle();
-				while (gtk_events_pending ())
-					 gtk_main_iteration ();
+//				while (gtk_events_pending ())
+//					 gtk_main_iteration ();
 				// running stops, so we scroll this cell into view
-				active_canvas->scroll_into_view(active_cell);
+//				active_canvas->scroll_into_view(active_cell);
+				set_next_into_view(active_cell);
 				}
 			else {
 #ifdef DEBUG
@@ -1820,7 +1910,11 @@ bool XCadabra::on_configure_event(GdkEventConfigure *cfg)
 bool XCadabra::on_delete_event(GdkEventAny* event)
 	{
 	if(quit_safeguard()) {
-		cdb.terminate();
+		// cdb.terminate();
+		// Ensure that the kernel is really gone, not just stuck in a loop.
+  		if(cdb.get_pid()!=0) 
+			if(kill(cdb.get_pid(), 0)==0)
+				kill(cdb.get_pid(), SIGKILL);
 		Gtk::Window::on_delete_event(event);
 		Gtk::Main::quit();
 		return false;
@@ -1904,8 +1998,11 @@ void XCadabra::remove_noninput_below(Glib::RefPtr<DataCell> dc)
 						(*it)->cell_type==DataCell::c_texcomment ||
 						(*it)->cell_type==DataCell::c_error) ) {
 				 for(unsigned int i=0; i<canvasses.size(); ++i) {
-					  canvasses[i]->remove_cell(*it);
-					  }
+#ifdef DEBUG
+					 std::cerr << "removing cell" << std::endl;
+#endif
+					 canvasses[i]->remove_cell(*it);
+					 }
 				it=datacells.erase(it);
 				}
 			return;
@@ -1946,6 +2043,9 @@ bool XCadabra::receive(modglue::ipipe& p)
 			ss >> help;
 			assert(id_to_datacell.find(help)!=id_to_datacell.end());
 			cp=id_to_datacell[help];
+#ifdef DEBUG
+			std::cerr << "original cell = " << cp->textbuf->get_text() << std::endl;
+#endif
 			remove_noninput_below(cp);
 			origcell=cp;
 			in_cell=true;
@@ -2186,9 +2286,10 @@ bool XCadabra::receive(modglue::ipipe& p)
 // FIXME: FOCUS
 //								  active_canvas->cell_grab_focus(datacells.back());
 								  }
-							 while (gtk_events_pending ())
-								  gtk_main_iteration ();
-							 active_canvas->scroll_into_view(active_cell);
+//							 while (gtk_events_pending ())
+//								  gtk_main_iteration ();
+//							 active_canvas->scroll_into_view(active_cell);
+							 set_next_into_view(active_cell);
 							 }
 						else { // this last cell is not an input cell; add a new input cell
 							Glib::RefPtr<DataCell> newcell(new DataCell(DataCell::c_input, ""));
@@ -2202,9 +2303,10 @@ bool XCadabra::receive(modglue::ipipe& p)
 							restarting_kernel=false;
 							active_canvas->cell_grab_focus(active_cell);
 							}
-						while (gtk_events_pending ())
-							gtk_main_iteration ();
-						active_canvas->scroll_into_view(active_cell);
+//						while (gtk_events_pending ())
+//							gtk_main_iteration ();
+//						active_canvas->scroll_into_view(active_cell);
+						set_next_into_view(active_cell);
 						// re-enable original cell (mark it non-running)
 						if(origcell) {
 							origcell->running=false;
@@ -2253,9 +2355,10 @@ bool XCadabra::receive(modglue::ipipe& p)
 								++it;
 								}
 							if(!running) { // put cell in view
-								 while (gtk_events_pending ())
-									  gtk_main_iteration ();
-								 active_canvas->scroll_into_view(active_cell);
+//								 while (gtk_events_pending ())
+//									  gtk_main_iteration ();
+//								 active_canvas->scroll_into_view(active_cell);
+								set_next_into_view(active_cell);
 								 }
 							}
 						}
@@ -2324,9 +2427,8 @@ bool XCadabra::receive(modglue::ipipe& p)
 			Glib::RefPtr<DataCell> newcell(new DataCell(DataCell::c_input));
 			add_cell(newcell, Glib::RefPtr<DataCell>());
 			show_cell(newcell);
-			while (gtk_events_pending ())
-				gtk_main_iteration ();
 
+			active_canvas->select_first_input_cell();
 			}
 		modified=false;
 		update_title();
@@ -2661,18 +2763,21 @@ std::string XCadabra::load(const std::string& fn, bool ignore_nonexistence)
 	struct stat buf;
 	int statres=lstat(fn.c_str(), &buf);
 	if(statres==-1) {
-		 switch(errno) {
-			  case EACCES:
-					return "Search permission denied.";
-			  case ELOOP:
-					return "Too many symbolic links.";
-			  case ENOENT:
-					if(ignore_nonexistence) return "";
-					else return "File does not exist.";
-			  default:
-					return "Error stat'ing file.";
-			  }
-		 }
+		switch(errno) {
+			case EACCES:
+				return "Search permission denied.";
+			case ELOOP:
+				return "Too many symbolic links.";
+			case ENOENT:
+				if(ignore_nonexistence) {
+					clear();
+					return "";
+					}
+				else return "File does not exist.";
+			default:
+				return "Error stat'ing file.";
+			}
+		}
 	
 	std::ifstream str(fn.c_str());
 	std::ostringstream err;
@@ -2871,10 +2976,10 @@ std::string XCadabra::load(const std::string& fn, bool ignore_nonexistence)
 		generic_error_popup(std::string(ex.what()));
 		}
 	show_all();
-	while (gtk_events_pending ())
-		gtk_main_iteration ();
+//	while (gtk_events_pending ())
+//		gtk_main_iteration ();
 	
-	active_canvas->select_first_input_cell();
+//	active_canvas->select_first_input_cell();
 
 	kernel_idle();
 	return "";
@@ -2883,7 +2988,11 @@ std::string XCadabra::load(const std::string& fn, bool ignore_nonexistence)
 void XCadabra::on_file_quit()
 	{
 	if(quit_safeguard()) {
-		cdb.terminate();
+		// cdb.terminate();
+		// Ensure that the kernel is really gone, not just stuck in an infinite loop.
+  		if(cdb.get_pid()!=0) 
+			if(kill(cdb.get_pid(), 0)==0)
+				kill(cdb.get_pid(), SIGKILL);
 		Gtk::Main::quit();
 		}
 	}
@@ -2938,10 +3047,11 @@ void XCadabra::on_edit_insert_tex_above()
 	Glib::RefPtr<DataCell> newcell(new DataCell(DataCell::c_tex, "[empty TeX cell]"));
 	action_add(Glib::RefPtr<ActionBase>(new ActionAddCell(newcell, active_cell->datacell, true)));
 	show_cell(newcell);
-	while (gtk_events_pending ())
-		 gtk_main_iteration ();
+//	while (gtk_events_pending ())
+//		 gtk_main_iteration ();
 	active_canvas->cell_grab_focus(newcell);
-	active_canvas->scroll_into_view(newcell);
+	set_next_into_view(active_cell);
+//	active_canvas->scroll_into_view(newcell);
 	}
 
 void XCadabra::on_edit_insert_tex_below()
@@ -2949,10 +3059,11 @@ void XCadabra::on_edit_insert_tex_below()
 	Glib::RefPtr<DataCell> newcell(new DataCell(DataCell::c_tex, "[empty TeX cell]"));
 	action_add(Glib::RefPtr<ActionBase>(new ActionAddCell(newcell, active_cell->datacell, false)));
 	show_cell(newcell);
-	while (gtk_events_pending ())
-		 gtk_main_iteration ();
+//	while (gtk_events_pending ())
+//		 gtk_main_iteration ();
 	active_canvas->cell_grab_focus(newcell);
-	active_canvas->scroll_into_view(newcell);
+	set_next_into_view(active_cell);
+//	active_canvas->scroll_into_view(newcell);
 	}
 
 void XCadabra::on_edit_insert_input_above()
@@ -2960,10 +3071,11 @@ void XCadabra::on_edit_insert_input_above()
 	Glib::RefPtr<DataCell> newcell(new DataCell(DataCell::c_input, ""));
 	action_add(Glib::RefPtr<ActionBase>(new ActionAddCell(newcell, active_cell->datacell, true)));
 	show_cell(newcell);
-	while (gtk_events_pending ())
-		 gtk_main_iteration ();
+//	while (gtk_events_pending ())
+//		 gtk_main_iteration ();
 	active_canvas->cell_grab_focus(newcell);
-	active_canvas->scroll_into_view(newcell);
+	set_next_into_view(active_cell);
+//	active_canvas->scroll_into_view(newcell);
 	}
 
 void XCadabra::on_edit_insert_input_below()
@@ -2971,10 +3083,11 @@ void XCadabra::on_edit_insert_input_below()
 	Glib::RefPtr<DataCell> newcell(new DataCell(DataCell::c_input, ""));
 	action_add(Glib::RefPtr<ActionBase>(new ActionAddCell(newcell, active_cell->datacell, false)));
 	show_cell(newcell);
-	while (gtk_events_pending ())
-		 gtk_main_iteration ();
+//	while (gtk_events_pending ())
+//		 gtk_main_iteration ();
 	active_canvas->cell_grab_focus(newcell);
-	active_canvas->scroll_into_view(newcell);
+	set_next_into_view(active_cell);
+//	active_canvas->scroll_into_view(newcell);
 	}
 
 void XCadabra::on_edit_insert_section_above()
@@ -2990,6 +3103,11 @@ void XCadabra::on_edit_insert_section_above()
 void XCadabra::on_edit_remove_cell()
 	{
 	Glib::RefPtr<DataCell> dc=active_cell->datacell;
+
+	// Prevent removal of the first cell of the notebook.
+	if(std::find(datacells.begin(), datacells.end(), dc)==datacells.begin())
+		return;
+
 	if(dc->cell_type!=DataCell::c_input && dc->cell_type!=DataCell::c_tex) return;
 
 	action_add(Glib::RefPtr<ActionBase>(new ActionRemoveCell(dc)));
@@ -3079,6 +3197,21 @@ void XCadabra::on_settings_font_size(int num)
 		canvasses[i]->redraw_cells();
 	}
 
+void XCadabra::on_settings_brain(int num)
+	{
+	if(brain_wired==num) return;
+
+	brain_wired=num;
+
+	std::string res=save_config();
+	if(res.size()>0) {
+		 Gtk::MessageDialog md("Error");
+		 md.set_secondary_text(res);
+		 md.set_type_hint(Gdk::WINDOW_TYPE_HINT_DIALOG);
+		 md.run();
+		 }
+	}
+
 void XCadabra::on_tutorial_open(unsigned int num)
 	{
 	Gtk::MessageDialog md("Tutorial information");
@@ -3102,6 +3235,24 @@ void XCadabra::on_clipboard_clear()
 	{
 	}
 
+void XCadabra::set_next_into_view(VisualCell *vc)
+	{
+	to_scroll_to=vc;
+	// Activate the on_idle handler so that we can handle this request.
+	Glib::signal_idle().connect( sigc::mem_fun(*this, &XCadabra::on_idle) );
+	}
+
+bool XCadabra::on_idle()
+	{
+	if(active_canvas!=0) {
+		if(to_scroll_to!=0) {
+			active_canvas->scroll_into_view(to_scroll_to, false);
+			to_scroll_to=0;
+			}
+		}
+	return false;
+	}
+
 std::string XCadabra::save_config() const
 	{
 	std::string defname=getenv("HOME");
@@ -3112,6 +3263,7 @@ std::string XCadabra::save_config() const
 
 	conf << "# XCadabra configuration file version 1.0" << std::endl;
 	conf << "font_step:=" << font_step << std::endl;
+	conf << "brain_wired:=" << brain_wired << std::endl;
 	conf.close();
 	return "";
 	}
@@ -3143,6 +3295,15 @@ std::string XCadabra::load_config()
 						 }
 					font_step=tmp;
 					}
+			  else if(rl.substr(0,pos)=="brain_wired") {
+					int tmp=atoi(rl.substr(pos+2).c_str());
+					if(tmp<0 || tmp>1) {
+						 str << "Out-of-bounds value for " << rl.substr(0,pos) 
+							  << " in ~/.xcadabra on line " << line << std::endl;
+						 return str.str();
+						 }
+					brain_wired=tmp;
+					}
 			  else {
 					str << "Unknown identifier " << rl.substr(0,pos) 
 						 << " in ~/.xcadabra line " << line << std::endl;
