@@ -369,9 +369,11 @@ void algorithm::apply(unsigned int lue, bool multiple, bool until_nochange, bool
 //			debugout << "no marks" << std::endl;
 			iterator dummy=tr.end();
 			++number_of_calls;
-			result_t res=apply(dummy);
-			if(is_output_module || res!=l_error)
-				global_success=g_applied;
+			if( can_apply(dummy) ) {
+				result_t res=apply(dummy);
+				if(is_output_module || res!=l_error)
+					global_success=g_applied;
+				}
 			}
 		discard_command_node=true;
 		}
@@ -1610,7 +1612,7 @@ algorithm::range_vector_t::iterator algorithm::find_arg_superset(range_vector_t&
 bool algorithm::is_single_term(iterator it)
 	{
 	if(*it->name!="\\prod" && *it->name!="\\sum" && *it->name!="\\asymimplicit" && *it->name!="\\comma" 
-		&& *it->name!="\\equals") {
+		&& *it->name!="\\equals" && *it->name!="\\arrow") {
 		if(tr.is_valid(tr.parent(it))) {
 			if(*tr.parent(it)->name=="\\sum" || *tr.parent(it)->name=="\\expression" || tr.parent(it)->is_command())
 				return true;
@@ -1620,6 +1622,19 @@ bool algorithm::is_single_term(iterator it)
 	return false;
 	}
 
+bool algorithm::is_nonprod_factor_in_prod(iterator it)
+	{
+	if(*it->name!="\\prod" && *it->name!="\\sum" && *it->name!="\\asymimplicit" && *it->name!="\\comma" 
+		&& *it->name!="\\equals") {
+		if(tr.is_valid(tr.parent(it))) {
+			if(*tr.parent(it)->name=="\\prod")
+				return true;
+			}
+//		else return true;
+		}
+	return false;
+	}
+
 bool algorithm::prod_wrap_single_term(iterator& it)
 	{
 	if(is_single_term(it)) {
@@ -1652,6 +1667,60 @@ bool algorithm::prod_unwrap_single_term(iterator& it)
 	return false;
 	}
 
+bool algorithm::separated_by_derivative(iterator i1, iterator i2, iterator check_dependence) const
+	{
+	iterator lca = tr.lowest_common_ancestor(i1, i2);
+
+	// Walk up the tree from the first node until the LCA, flag any derivatives
+	// with which we do not commute.
+
+	struct {
+		bool operator()(exptree& tr, iterator walk, iterator lca, iterator check_dependence) {
+		   do {
+				walk=exptree::parent(walk);
+				if(walk == lca) break;
+				const Derivative *der=properties::get<Derivative>(walk);
+				if(der) {
+					if(tr.is_valid(check_dependence) ) {
+						const DependsBase *dep = properties::get_composite<DependsBase>(check_dependence);
+						if(dep) {
+							exptree deps=dep->dependencies(check_dependence);
+							sibling_iterator depobjs=deps.begin(deps.begin());
+							while(depobjs!=deps.end(deps.begin())) {
+								if(walk->name == depobjs->name) {
+									return true;
+									}
+								else {
+									// compare all indices
+									sibling_iterator indit=tr.begin(walk);
+									while(indit!=tr.end(walk)) {
+										if(indit->is_index()) {
+											if(subtree_exact_equal(indit, depobjs))
+												return true;
+											}
+										++indit;
+										}
+									}
+								++depobjs;
+								} 
+							return false; // Dependence found but not relevant here.
+							}
+						else return false; // No dependence property found at all.
+						}
+					else return true; // Should not check for dependence.
+					}
+				} while(walk != lca);
+			return false;
+		   }
+	} one_run;
+	
+	if(one_run(tr, i1, lca, check_dependence)) return true;
+	if(one_run(tr, i2, lca, check_dependence)) return true;
+
+	return false;
+	}
+
+
 void cleanup_expression(exptree& tr)
 	{
 	exptree::iterator it=tr.begin();
@@ -1720,12 +1789,16 @@ void cleanup_nests_below(exptree&tr, exptree::iterator it, bool ignore_bracket_t
 
 	while(now!=stop) {
 		cleanup_nests(tr, now, ignore_bracket_types);
+// Iterators should always be valid when we return here, so this test is not required.	  
+//		if(tr.is_valid(now)==false)
+//			break;
 		++now;
 		}
 	}
 
 void cleanup_nests(exptree&tr, exptree::iterator &it, bool ignore_bracket_types)
 	{
+	if(!tr.is_valid(it)) return;
 	if(!tr.is_valid(tr.parent(it))) return;
 //	tr.print_recursive_treeform(txtout, tr.begin());
 	if(*(it->name)=="\\prod") {
@@ -1756,6 +1829,7 @@ void cleanup_nests(exptree&tr, exptree::iterator &it, bool ignore_bracket_types)
 				++terms;
 				}
 			tr.flatten(it);
+			// FIXME: this is dangerous:
 			it=tr.parent(tr.erase(it));
 			}
 		return;
