@@ -1,7 +1,7 @@
 /* 
 
 	Cadabra: a field-theory motivated computer algebra system.
-	Copyright (C) 2001-2009  Kasper Peeters <kasper.peeters@aei.mpg.de>
+	Copyright (C) 2001-2010  Kasper Peeters <kasper.peeters@aei.mpg.de>
 
    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -80,13 +80,15 @@ void exptree_output::setup_handlers(bool infix)
 				printers_["\\factorial"]    =&create<print_factorial>;
 				printers_["\\sequence"]     =&create<print_sequence>;
 				printers_["\\commutator"]   =&create<print_commutator>;
+				printers_["\\anticommutator"]=&create<print_commutator>;
 				printers_["\\indexbracket"] =&create<print_indexbracket>;
 				printers_["\\pow"]          =&create<print_pow>;
 				printers_["\\sqrt"]         =&create<print_sqrt>;
 				if(output_format==out_xcadabra) {
 					printers_prop_["Tableau"]       = &create<print_tableau>;
 					printers_prop_["FilledTableau"] = &create<print_filled_tableau>;
-//					printers_prop_["Derivative"]    = &create<print_derivative>;
+					printers_prop_["Derivative"]    = &create<print_derivative>;
+					printers_prop_["PartialDerivative"]    = &create<print_derivative>;
 					}
 				break;
 			}
@@ -127,7 +129,7 @@ std::auto_ptr<node_base_printer> exptree_output::get_printer(exptree::iterator i
 	printmap_prop_t::iterator pp=printers_prop_.begin();
 	while(pp!=printers_prop_.end()) {
 		properties::registered_property_map_t::iterator pit=
-			properties::registered_properties.find((*pp).first);
+			properties::registered_properties.store.find((*pp).first);
 
 		const property_base *aprop=pit->second();
 		bool ret=properties::has(aprop, it);
@@ -312,18 +314,20 @@ void print_pow::print_infix(std::ostream& str, exptree::iterator it)
 
 	bool close_bracket=false;
 	sibling_iterator st=tr.begin(it);
-	while(st!=tr.end(it)) {
-		if(*st->name=="\\sum") {
-			str << "(";
-			close_bracket=true;
-			break;
-			}
-		if(tr.number_of_children(st)!=tr.number_of_indices(st)) {
-			str << "(";
-			close_bracket=true;
-			break;
-			 }
-		++st;
+
+	// The first argument of \pow is to be examined for special
+	// cases which require brackets around the entire argument.
+	if(*st->name=="\\sum") {
+		str << "(";
+		close_bracket=true;
+		}
+	else if(tr.number_of_children(st)!=tr.number_of_indices(st)) {
+		str << "(";
+		close_bracket=true;
+		}
+	else if(*st->multiplier!=1) {
+		str << "(";
+		close_bracket=true;
 		}
 
 	sibling_iterator ch=tr.begin(it);
@@ -337,9 +341,10 @@ void print_pow::print_infix(std::ostream& str, exptree::iterator it)
 		str << "}";
 		}
 	else {
-		str << "**";
+		str << "**{";
 		++ch;
 		parent.get_printer(ch)->print_infix(str, ch);
+		str << "}";
 		}
 	}
 
@@ -409,7 +414,8 @@ void print_sum::do_actual_print(std::ostream& str, exptree::iterator it)
 		print_multiplier(str, it);
 
 	iterator par=tr.parent(it);
-	if(tr.number_of_children(par)>1) { // for a single argument, the parent already takes care of the brackets
+	if(tr.number_of_children(par) - tr.number_of_direct_indices(par)>1) { 
+      // for a single argument, the parent already takes care of the brackets
 		if(*it->multiplier!=1 || (tr.is_valid(par) && *par->name!="\\expression")) {
 			// test whether we need extra brackets
 			close_bracket=!children_have_brackets(it);
@@ -693,24 +699,77 @@ void print_filled_tableau::print_infix(std::ostream& str, exptree::iterator it)
 	}
 
 
-// print_derivative::print_derivative(exptree_output& eo)
-// 	: node_printer(eo)
-// 	{
-// 	}
-// 
-// void print_derivative::print_infix(std::ostream& str, exptree::iterator it)
-// 	{
-// 	if(*it->multiplier!=1) 
-// 		print_multiplier(str, it);
-// 	sibling_iterator sib=tr.begin(it);
-// 	str << *it->name;
-// 	while(sib!=tr.end(it)) {
-// 		 if(sib->is_index()) {
-// 			  
-// 			  }
-// 		 ++sib;
-// 		 }
-// 	}
+print_derivative::print_derivative(exptree_output& eo)
+	: node_printer(eo)
+	{
+	}
+
+void print_derivative::print_infix(std::ostream& str, exptree::iterator it)
+	{
+	if(*it->multiplier!=1) 
+		print_multiplier(str, it);
+
+	const LaTeXForm *lf=properties::get<LaTeXForm>(it);
+	bool needs_extra_brackets=false;
+
+	if(parent.output_format==exptree_output::out_xcadabra) {
+		const Accent *ac=properties::get<Accent>(it);
+		if(!ac) { // accents should never get additional curly brackets, {\bar}{g} does not print.
+			sibling_iterator sib=tr.begin(it);
+			while(sib!=tr.end(it)) {
+				if(sib->is_index()) 
+					needs_extra_brackets=true;
+				++sib;
+				}
+			}
+		
+		if(needs_extra_brackets) str << "{"; // to prevent double sup/sub script errors
+		if(lf) str << lf->latex;
+		else   str << texify(*it->name);
+		if(needs_extra_brackets) str << "}";
+		}
+	else str << *it->name;
+
+	sibling_iterator idx=tr.begin(it);
+	int count=0;
+	previous_parent_rel_=str_node::p_none;
+	
+	while(idx!=tr.end(it)) {
+		sibling_iterator nxt=idx;
+		++nxt;
+
+		if(idx->is_index()) {
+			if(idx->fl.parent_rel!=previous_parent_rel_) {
+				print_parent_rel(str, idx->fl.parent_rel, idx==tr.begin(it));
+				str << "{";
+				previous_parent_rel_=idx->fl.parent_rel;
+				parent.get_printer(idx)->print_infix(str, idx);
+				}
+			else
+				parent.get_printer(idx)->print_infix(str, idx);
+
+			if(nxt==tr.end(it) || nxt->fl.parent_rel!=previous_parent_rel_)
+				str << "}";
+			else 
+				str << " ";
+			}
+		else {
+			str << "{";
+			if(*idx->name=="\\prod" || *idx->name=="\\sum")
+				str << "(";
+
+			parent.get_printer(idx)->print_infix(str, idx);
+
+			if(*idx->name=="\\prod" || *idx->name=="\\sum")
+				str << ")";
+			str << "}";
+			}
+		++idx;
+		++count;
+		}
+	if(parent.output_format==exptree_output::out_xcadabra)
+		str << "\\, ";
+	}
 
 print_sequence::print_sequence(exptree_output& eo)
 	: node_printer(eo)
@@ -738,22 +797,33 @@ void print_commutator::print_infix(std::ostream& str, exptree::iterator it)
 		parent.output_format!=exptree_output::out_texmacs ) 
 		prettyform=false;
 	else prettyform=true;
+	
+	bool anticommutator=true;
+	if(*it->name=="\\commutator") anticommutator=false;
 
 	if(*it->multiplier!=1) 
 		print_multiplier(str, it);
 	sibling_iterator ch=tr.begin(it);
-	if(prettyform)	str << "[";
-	else str << "\\commutator{";
+	if(prettyform)	{
+		if(anticommutator) str << "\\{";
+		else               str << "[";
+		}
+	else {
+		str << *it->name << "{";
+		}
 
 	parent.get_printer(ch)->print_infix(str, ch);
+
 	if(prettyform)	str << ",";
 	else str << "}{";
 
 	++ch;
 	parent.get_printer(ch)->print_infix(str, ch);
 
-	parent.get_printer(ch)->print_infix(str, ch);
-	if(prettyform)	str << "]";
+	if(prettyform)	{
+		if(anticommutator) str << "\\}";
+		else               str << "]";
+		}
 	else str << "}";
 	}
 
@@ -930,12 +1000,10 @@ node_printer::node_printer(exptree_output& eo)
 
 void node_printer::print_infix(std::ostream& str, exptree::iterator it)
 	{
-//	if((*it).fl.mark && parent.highlight) str << "\033[1m";
-
 	// print multiplier and object name
 	if(*it->multiplier!=1)
 		print_multiplier(str, it);
-
+	
 	if(*it->name=="1") {
 		if(*it->multiplier==1) // this would print nothing altogether.
 			str << "1";
@@ -945,11 +1013,14 @@ void node_printer::print_infix(std::ostream& str, exptree::iterator it)
 	if(parent.output_format==exptree_output::out_xcadabra) {
 		const LaTeXForm *lf=properties::get<LaTeXForm>(it);
 		bool needs_extra_brackets=false;
-		sibling_iterator sib=tr.begin(it);
-		while(sib!=tr.end(it)) {
-			if(sib->is_index()) 
-				needs_extra_brackets=true;
-			++sib;
+		const Accent *ac=properties::get<Accent>(it);
+		if(!ac) { // accents should never get additional curly brackets, {\bar}{g} does not print.
+			sibling_iterator sib=tr.begin(it);
+			while(sib!=tr.end(it)) {
+				if(sib->is_index()) 
+					needs_extra_brackets=true;
+				++sib;
+				}
 			}
 		
 		if(needs_extra_brackets) str << "{"; // to prevent double sup/sub script errors
@@ -977,110 +1048,81 @@ void node_printer::print_children(std::ostream& str, exptree::iterator it, int s
 	previous_bracket_   =str_node::b_invalid;
 	previous_parent_rel_=str_node::p_none;
 
-	if(parent.output_format==exptree_output::out_mathematica && isdelta) { // Mathematica
-		str << "{";
-		exptree::sibling_iterator ch=tr.begin(it);
-		while(ch!=tr.end(it)) {
-			if(ch!=tr.begin(it)) str << "," << nbsp;
-			parent.get_printer(ch)->print_infix(str, ch);
-			++ch;
-			++ch;
-			}
-		str << "}," << nbsp << "{";
-		ch=tr.begin(it);
-		while(ch!=tr.end(it)) {
-			if(ch!=tr.begin(it)) str << "," << nbsp;
-			++ch;
-			parent.get_printer(ch)->print_infix(str, ch);
-			++ch;
-			}
-		str << "}";
-		}
-	else if(parent.output_format==exptree_output::out_maple && isweyl) { // Maple
-		exptree::sibling_iterator ch=tr.begin(it);
-		while(ch!=tr.end(it)) {
-			if(ch!=tr.begin(it)) str << "," << nbsp;
-			if(ch->fl.parent_rel==str_node::p_sub) 
-				str << "-";
-			parent.get_printer(ch)->print_infix(str, ch);
-			++ch;
-			}
-		}
-	else { // Normal (non-maple, non-mathematica) output
-		int number_of_nonindex_children=0;
-		int number_of_index_children=0;
-		exptree::sibling_iterator ch=tr.begin(it);
-		while(ch!=tr.end(it)) {
-			if(ch->is_index()==false) {
+	int number_of_nonindex_children=0;
+	int number_of_index_children=0;
+	exptree::sibling_iterator ch=tr.begin(it);
+	while(ch!=tr.end(it)) {
+		if(ch->is_index()==false) {
+			++number_of_nonindex_children;
+			if(*ch->name=="\\prod")
 				++number_of_nonindex_children;
-				if(*ch->name=="\\prod")
-					++number_of_nonindex_children;
-				}
-			else ++number_of_index_children;
-			++ch;
 			}
-
-		ch=tr.begin(it);
-		ch+=skip;
-		unsigned int chnum=0;
-		while(ch!=tr.end(it)) {
-			current_bracket_   =(*ch).fl.bracket;
-			current_parent_rel_=(*ch).fl.parent_rel;
-			
-			if(current_bracket_!=str_node::b_none || previous_bracket_!=current_bracket_ || previous_parent_rel_!=current_parent_rel_) {
-				 if(parent.output_format==exptree_output::out_plain ||
-					 parent.output_format==exptree_output::out_xcadabra ||
-					 parent.output_format==exptree_output::out_texmacs )
-					print_parent_rel(str, current_parent_rel_, ch==tr.begin(it));
-				 if(parent.output_format!=exptree_output::out_reduce) {
+		else ++number_of_index_children;
+		++ch;
+		}
+	
+	ch=tr.begin(it);
+	ch+=skip;
+	unsigned int chnum=0;
+	while(ch!=tr.end(it)) {
+		current_bracket_   =(*ch).fl.bracket;
+		current_parent_rel_=(*ch).fl.parent_rel;
+		const Accent *is_accent=properties::get<Accent>(it);
+		
+		if(current_bracket_!=str_node::b_none || previous_bracket_!=current_bracket_ || previous_parent_rel_!=current_parent_rel_) {
+			if(parent.output_format==exptree_output::out_plain ||
+				parent.output_format==exptree_output::out_xcadabra ||
+				parent.output_format==exptree_output::out_texmacs )
+				print_parent_rel(str, current_parent_rel_, ch==tr.begin(it));
+			if(parent.output_format!=exptree_output::out_reduce) {
+				if(is_accent==0) 
 					print_opening_bracket(str, (number_of_nonindex_children>1 /* &&number_of_index_children>0 */ &&
 														 current_parent_rel_!=str_node::p_sub && 
-														 current_parent_rel_!=str_node::p_super ? str_node::b_round:current_bracket_), current_parent_rel_);
-					 }
-				else
-					str << zwnbsp << "(" << zwnbsp;
+														 current_parent_rel_!=str_node::p_super ? str_node::b_round:current_bracket_), 
+												 current_parent_rel_);
+				else str << "{";
 				}
-			if(parent.output_format==exptree_output::out_mathematica && getenv("CDB_MATH_COMPAC")==0) {
-				if(ch!=tr.begin(it)) {
-					if(chnum==2 && isweyl)
-						str << "}," << nbsp << "{";
-					else
-						str << "," << nbsp;
-					}
-				}
-			else if(parent.output_format==exptree_output::out_reduce) {
-				if(ch!=tr.begin(it))
-					str << "," << zwnbsp;
-				}
-			parent.get_printer(ch)->print_infix(str, ch);
-//			if((*it).fl.mark && parent.highlight) str << "\033[1m"; 
-			++ch;
-			if(ch==tr.end(it) || current_bracket_!=str_node::b_none ||
-				current_bracket_!=(*ch).fl.bracket || current_parent_rel_!=(*ch).fl.parent_rel) {
-				if(parent.output_format!=exptree_output::out_reduce)
-					 print_closing_bracket(str,  (number_of_nonindex_children>1 /* &&number_of_index_children>0 */ && 
-														  current_parent_rel_!=str_node::p_sub && 
-														  current_parent_rel_!=str_node::p_super ? str_node::b_round:current_bracket_), current_parent_rel_);
-				else
-					str << ")";
-				}
-			else if(parent.output_format==exptree_output::out_plain 
-					  || parent.output_format==exptree_output::out_xcadabra
-					  || parent.output_format==exptree_output::out_maple
-					  || parent.output_format==exptree_output::out_texmacs)
-				str << nbsp;
-			
-			previous_bracket_=current_bracket_;
-			previous_parent_rel_=current_parent_rel_;
-			++chnum;
+			else
+				str << zwnbsp << "(" << zwnbsp;
 			}
+		if(parent.output_format==exptree_output::out_mathematica && getenv("CDB_MATH_COMPAC")==0) {
+			if(ch!=tr.begin(it)) {
+				if(chnum==2 && isweyl)
+					str << "}," << nbsp << "{";
+				else
+					str << "," << nbsp;
+				}
+			}
+		else if(parent.output_format==exptree_output::out_reduce) {
+			if(ch!=tr.begin(it))
+				str << "," << zwnbsp;
+			}
+		parent.get_printer(ch)->print_infix(str, ch);
+//			if((*it).fl.mark && parent.highlight) str << "\033[1m"; 
+		++ch;
+		if(ch==tr.end(it) || current_bracket_!=str_node::b_none ||
+			current_bracket_!=(*ch).fl.bracket || current_parent_rel_!=(*ch).fl.parent_rel) {
+			if(parent.output_format!=exptree_output::out_reduce) {
+				if(is_accent==0) 
+					print_closing_bracket(str,  (number_of_nonindex_children>1 /* &&number_of_index_children>0 */ && 
+														  current_parent_rel_!=str_node::p_sub && 
+														  current_parent_rel_!=str_node::p_super ? str_node::b_round:current_bracket_), 
+												 current_parent_rel_);
+				else str  << "}";
+				}
+			else
+				str << ")";
+			}
+		else if(parent.output_format==exptree_output::out_plain 
+				  || parent.output_format==exptree_output::out_xcadabra
+				  || parent.output_format==exptree_output::out_maple
+				  || parent.output_format==exptree_output::out_texmacs)
+			str << nbsp;
+		
+		previous_bracket_=current_bracket_;
+		previous_parent_rel_=current_parent_rel_;
+		++chnum;
 		}
-	if(parent.output_format==exptree_output::out_mathematica && 
-		it->fl.parent_rel==str_node::p_none && tr.number_of_children(it)>0)
-		str << "]";
-	if(parent.output_format==exptree_output::out_maple && isweyl)
-		str << "]";
-//	if((*it).fl.mark && parent.highlight) str << "\033[0m";	
 	}
 
 void node_printer::print_multiplier(std::ostream& str, exptree::iterator it)
@@ -1145,7 +1187,10 @@ void node_printer::print_multiplier(std::ostream& str, exptree::iterator it)
 void node_printer::print_opening_bracket(std::ostream& str, str_node::bracket_t br, str_node::parent_rel_t pr)
 	{
 	switch(br) {
-		case str_node::b_none:   str << "{";   break;
+		case str_node::b_none: 
+			if(parent.output_format==exptree_output::out_xcadabra && pr==str_node::p_none) str << "(";  
+			else                                                                           str << "{";
+			break;
 		case str_node::b_pointy: str << "\\<"; break;
 		case str_node::b_curly:  str << "\\{"; break;
 		case str_node::b_round:  str << "(";   break;
@@ -1158,7 +1203,10 @@ void node_printer::print_opening_bracket(std::ostream& str, str_node::bracket_t
 void node_printer::print_closing_bracket(std::ostream& str, str_node::bracket_t br, str_node::parent_rel_t pr)
 	{
 	switch(br) {
-		case str_node::b_none:   str << "}";   break;
+		case str_node::b_none:   
+			if(parent.output_format==exptree_output::out_xcadabra && pr==str_node::p_none) str << ")";  
+			else                                                                           str << "}";
+			break;
 		case str_node::b_pointy: str << "\\>"; break;
 		case str_node::b_curly:  str << "\\}"; break;
 		case str_node::b_round:  str << ")";   break;
