@@ -1,7 +1,7 @@
 /* 
 
 	Cadabra: a field-theory motivated computer algebra system.
-	Copyright (C) 2001-2009  Kasper Peeters <kasper.peeters@aei.mpg.de>
+	Copyright (C) 2001-2010  Kasper Peeters <kasper.peeters@aei.mpg.de>
 
    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -173,7 +173,7 @@ manipulator::manipulator()
 
 	// numerical
 	numerical::register_properties();
-	algorithms["@numerical_flatten"] = new algo_info(&create<numerical_flatten>);
+	algorithms["@numerical_flatten"] = new algo_info(&create<numerical::numerical_flatten>);
 
 	// properties
 	algorithms["@props"]          =new algo_info(&create<extract_properties>); // internal: not documented yet
@@ -430,6 +430,8 @@ bool manipulator::handle_input()
 		   again:
 			while(pos>0 && isblank(oneline[pos]))
 				--pos;
+			if(oneline[pos]==';' && pos>0 && oneline[pos-1]=='\"')
+				--pos; // hack to allow <"filename"; with the semi-colon at the end
 			switch(oneline[pos]) {
 				case '\"':{
 					int quotepos=pos;
@@ -888,8 +890,8 @@ exptree::iterator manipulator::handle_active_nodes_(exptree::iterator original_e
 			return expressions.end();
 			}
 		else if(*it->name=="@properties") {
-			 properties::registered_property_map_t::iterator pit=properties::registered_properties.begin();
-			 while(pit!=properties::registered_properties.end()) {
+			 properties::registered_property_map_t::iterator pit=properties::registered_properties.store.begin();
+			 while(pit!=properties::registered_properties.store.end()) {
 				  if(eo.output_format==exptree_output::out_xcadabra)
 						txtout << "<property>\n";
 				  txtout << "::" << pit->first << std::endl;
@@ -998,7 +1000,8 @@ exptree::iterator manipulator::handle_active_nodes_(exptree::iterator original_e
 					collect_after=to_long(*args->multiplier);
 				iterator newit=run_procedure(procit,collect_after);
 				expressions.erase_expression(original_expression);
-				txtout << "procedure completed." << std::endl;
+				if(!nowarnings)
+					txtout << "procedure completed." << std::endl;
 				return newit;
 				}
 			}
@@ -1052,6 +1055,9 @@ exptree::iterator manipulator::handle_active_nodes_(exptree::iterator original_e
 
 exptree::iterator manipulator::run_procedure(exptree::iterator proc, long collect_after)
 	{
+	bool remember_silentfail=silentfail;
+	silentfail=true;
+
 	collect_terms collector(expressions, expressions.end());
 	long termcount=0;
 	long collectcount=0;
@@ -1110,9 +1116,11 @@ exptree::iterator manipulator::run_procedure(exptree::iterator proc, long collec
 			}
 		// statistics output
 		expired_so_far.stop();
-		txtout << "term " << termcount << " of " << totalterms;
-		txtout << " remains " << expired_so_far.seconds()*(float)(totalterms-termcount)/(float)(termcount)/60
-				 << " min" << std::endl;
+		if(!nowarnings) {
+			txtout << "term " << termcount << " of " << totalterms;
+			txtout << " remains " << expired_so_far.seconds()*(float)(totalterms-termcount)/(float)(termcount)/60
+					 << " min" << std::endl;
+			}
 		expired_so_far.start();
 
 		// replace term with new current expression.
@@ -1151,8 +1159,13 @@ exptree::iterator manipulator::run_procedure(exptree::iterator proc, long collec
 	if(collected)
 		cleanup_expression(expressions, act); // to remove possible zeroes
 	last_used_equation_number=backup_last_used;
-	for(unsigned int i=0; i<timers.size(); ++i)
-		txtout << "line " << i << " total " << timers[i] << std::endl;
+	if(!nowarnings) 
+		for(unsigned int i=0; i<timers.size(); ++i) 
+			txtout << "line " << i << " total " << timers[i] << std::endl;
+
+	// Restore settings for warning/failure messages.
+	silentfail=remember_silentfail;
+
 	return expressions.active_expression(expressions.equation_by_number(last_used_equation_number));
 	}
 
