@@ -728,8 +728,8 @@ algorithm::result_t eliminate_kronecker::apply(iterator& st)
 			sibling_iterator ii1=tr.begin(it);
 			sibling_iterator ii2=ii1; ++ii2;
 			if(subtree_compare(ii1, ii2, 1, false, true)==0) { // a self-contracted Kronecker delta
-				 const Integer *itg1=properties::get<Integer>(ii1);
-				 const Integer *itg2=properties::get<Integer>(ii2);
+				 const numerical::Integer *itg1=properties::get<numerical::Integer>(ii1);
+				 const numerical::Integer *itg2=properties::get<numerical::Integer>(ii2);
 				 if(itg1 && itg2 && ii1->is_rational()==false && ii2->is_rational()==false) {
 					  if(itg1->from.begin()!=itg1->from.end() && itg2->from.begin()!=itg2->from.end()) {
 							if(itg1->difference.begin()->name==onept) {
@@ -749,23 +749,34 @@ algorithm::result_t eliminate_kronecker::apply(iterator& st)
 				 sibling_iterator oi=tr.begin(st);
 				 ++looping;
 				 // iterate over all factors in the product
+				 bool doing1=false;
+				 bool doing2=false;
 				 while(!replaced && oi!=tr.end(st)) {
 					  if(oi!=it) { // this is not the delta node
 							// compare delta indices with all indices of this object
 							exptree::index_iterator ind=tr.begin_index(oi);
 							while(ind!=tr.end_index(oi)) {
-								 if(ii1->is_rational()==false && subtree_compare(ind, ii1, 1, false, true)==0) {
-									  tr.replace_index(ind, ii2)->fl.parent_rel=ii2->fl.parent_rel; 
-									  replaced=true;
-									  break;
-									  }
-								 else if(ii2->is_rational()==false && subtree_compare(ind, ii2, 1, false, true)==0) {
-									  tr.replace_index(ind, ii1)->fl.parent_rel=ii1->fl.parent_rel;
-									  replaced=true;
-									  break;
-									  }
-								 ++ind;
-								 }
+								exptree::index_iterator nxt=ind;
+								++nxt;
+								if(ii1->is_rational()==false && subtree_compare(ind, ii1, 1, false, true)==0 ) {
+									if(! (replaced && doing2) ) {
+										tr.replace_index(ind, ii2)->fl.parent_rel=ii2->fl.parent_rel; 
+										replaced=true;
+										doing1=true;
+										}
+									// cannot 'break' here because that would miss cases when the 
+									// delta multiplies a sum.
+									}
+								else if(ii2->is_rational()==false && subtree_compare(ind, ii2, 1, false, true)==0) {
+									if(! (replaced && doing1) ) {
+										tr.replace_index(ind, ii1)->fl.parent_rel=ii1->fl.parent_rel;
+										replaced=true;
+										doing2=true;
+										}
+									// no break here either.
+									}
+								ind=nxt;
+								}
 							}
 					  if(!replaced) 
 							++oi;
@@ -862,7 +873,7 @@ bool reduce_gendelta::one_step_(sibling_iterator dl)
 //	tr.print_recursive_treeform(debugout, dl, num) << std::endl;}
 	// FIXME: use properties for the dimension!
 //	txtout << *dl->multiplier << std::endl;
-	const Integer *itg=properties::get<Integer>(up);
+	const numerical::Integer *itg=properties::get<numerical::Integer>(up);
 	int dim;
 	if(itg) {
 		const nset_t::iterator onept=name_set.insert("1").first;
@@ -1122,7 +1133,7 @@ algorithm::result_t dualise_tensor::apply(iterator& st)
 		iterator tmpit=rep.append_child(eps,  dummy.begin());
 		generated_indices.insert(index_map_t::value_type(dummy, tmpit));
 		tmpit->fl.bracket=str_node::b_none;
-		if(dums->position_free)
+		if(dums->position_type==Indices::free)
 			 tmpit->fl.parent_rel=str_node::p_sub;
 		else
 			 tmpit->fl.parent_rel=str_node::p_super;
@@ -1550,6 +1561,8 @@ algorithm::result_t pintegrate::apply(iterator& it)
 
 				// Clean up nests of \partial if present.
 //				iterator tmpit=tr.begin(newdiff);
+//				iterator tmp=newdiff;
+//				cleanup_nests(tr, tmp);
 				cleanup_nests_below(tr, (iterator)newdiff);
 				}
 
@@ -1596,6 +1609,8 @@ algorithm::result_t pintegrate::apply(iterator& it)
 
 				// Clean up nests of \partial if present.
 				cleanup_nests_below(tr, (iterator)newdiff);
+//				iterator tmp=newdiff;
+//				cleanup_nests(tr, tmp);
 //				sibling_iterator tmpit=tr.begin(newdiff);
 //				cleanup_nests(tr, tmpit);
 				}
@@ -1677,7 +1692,7 @@ algorithm::result_t unwrap::apply(iterator& it)
 				const DependsBase *dep=properties::get_composite<DependsBase>(factor);
 				if(dep!=0) {
 //					txtout << *factor->name << " depends" << std::endl;
-					exptree deps=dep->dependencies(it);
+					exptree deps=dep->dependencies(factor /* it */);
 					sibling_iterator depobjs=deps.begin(deps.begin());
 					while(depobjs!=deps.end(deps.begin())) {
 //						txtout << "?" << *it->name << " == " << *depobjs->name << std::endl;
