@@ -327,7 +327,7 @@ algorithm::result_t join::apply(iterator& st)
 	long number_of_dimensions=-1; // i.e. not known.
 	exptree::index_iterator firstind=tr.begin_index(gam1);
 	while(firstind!=tr.end_index(gam1)) {  // select the maximum value; FIXME: be more refined...
-		const Integer *ipr=properties::get<Integer>(firstind);
+		const numerical::Integer *ipr=properties::get<numerical::Integer>(firstind);
 		if(ipr) {
 			if(ipr->difference.begin()->is_integer()) {
 				number_of_dimensions=std::max(number_of_dimensions, to_long(*ipr->difference.begin()->multiplier));
@@ -342,7 +342,7 @@ algorithm::result_t join::apply(iterator& st)
 	if(number_of_dimensions!=-1) {
 		firstind=tr.begin_index(gam2);
 		while(firstind!=tr.end_index(gam2)) {  // select the maximum value; FIXME: be more refined...
-			const Integer *ipr=properties::get<Integer>(firstind);
+			const numerical::Integer *ipr=properties::get<numerical::Integer>(firstind);
 			if(ipr) {
 				if(ipr->difference.begin()->is_integer()) {
 					number_of_dimensions=std::max(number_of_dimensions, to_long(*ipr->difference.begin()->multiplier));
@@ -681,7 +681,7 @@ bool projweyl::can_apply(iterator st)
 		while(it!=tr.end(st)) {
 			const GammaMatrix *gm=properties::get<GammaMatrix>(it);
 			if(gm) {
-				const Integer *isint=properties::get<Integer>(tr.begin(gamma_loc_));
+				const numerical::Integer *isint=properties::get<numerical::Integer>(tr.begin(gamma_loc_));
 				if(isint) {
 					 if(isint->difference.begin()->is_rational()) {
 						  ++numgamma;
@@ -901,7 +901,7 @@ bool fierz::can_apply(iterator it)
 	// to find the gamma matrix, fermion and second fermi bilinear.
 
 	sibling_iterator sib=tr.begin(it);
-	const Integer *indit=0;
+	const numerical::Integer *indit=0;
 	while(sib!=tr.end(it)) {
 		const DiracBar *db=properties::get_composite<DiracBar>(sib);
 		if(db) {
@@ -920,7 +920,7 @@ bool fierz::can_apply(iterator it)
 			if(gmnxt) {
 //				txtout << "found gam" << std::endl;
 				// FIXME: should also work when there is a unit matrix in between.
-				indit=properties::get_composite<Integer>(ch.begin());
+				indit=properties::get_composite<numerical::Integer>(ch.begin());
 				indprop=properties::get_composite<Indices>(ch.begin());
 				if(!indit || !indprop) return false;
 				dim=to_long(*indit->difference.begin()->multiplier);
@@ -1072,7 +1072,7 @@ algorithm::result_t fierz::apply(iterator& it)
 			exptree newdum=get_dummy(indprop, &ind_free, &ind_dummy, &ind_added);
 			iterator loc1=cpyterm.append_child(locgam1, newdum.begin());
 			ind_added.insert(index_map_t::value_type(newdum, loc1));
-			if(indprop->position_free)
+			if(indprop->position_type==Indices::free)
 				loc1->fl.parent_rel=str_node::p_sub;
 			else
 				loc1->fl.parent_rel=str_node::p_super;
