@@ -147,224 +147,127 @@ bool RiemannTensor::parse(exptree& tr, exptree::iterator pat, exptree::iterator
 	}
 
 eliminate_vielbein::eliminate_vielbein(exptree& tr, iterator it)
-	: algorithm(tr, it)
+	: eliminate_converter(tr, it)
 	{
 	}
 
-void eliminate_vielbein::description() const
-	{
-	txtout << "Bla" << std::endl;
-	}
-
-bool eliminate_vielbein::can_apply(iterator it)
-	{
-	if(*it->name=="\\prod") return true;
-	return false;
-	}
-
-algorithm::result_t eliminate_vielbein::apply(iterator& it)
-	{
-	// Put arguments in canonical form.
-
-	sibling_iterator objs=args_begin();
-//	sibling_iterator vielb=objs;
-//	++vielb;
-	if(*objs->name!="\\comma")
-		objs=tr.wrap(objs, str_node("\\comma"));
-
-	index_map_t ind_dummy, ind_free;
-	classify_indices(it, ind_free, ind_dummy);
-
-	// Run over all factors, find vielbeine, figure out whether they can
-	// be used to turn the indices on the other tensors to preferred type.
-
-	sibling_iterator fit=tr.begin(it);
-	while(fit!=tr.end(it)) {
-		const Vielbein         *vb=properties::get<Vielbein>(fit);
-		const InverseVielbein *ivb=properties::get<InverseVielbein>(fit);
-		bool replaced=false;
-		if(vb || ivb) {
-	      sibling_iterator ind1=tr.begin(fit), ind2=ind1; // the 1st and 2nd index of the vielbein
-			++ind2;
-			// The contracted index can be either ind1 or ind2. Hence two similar loops:
-
-			// 1st index to 2nd index conversion?
-			std::pair<index_map_t::const_iterator,index_map_t::const_iterator> 
-				 locs= ind_dummy.equal_range(exptree(ind1));
-			while(locs.first!=locs.second) {
-				 if(locs.first->second!=(iterator)ind1) {
-					  // Does this index sit on an object in the "preferred form" list?
-					  iterator par=tr.parent(locs.first->second);
-					  sibling_iterator prefit=tr.begin(objs);
-					  while(prefit!=tr.end(objs)) {
-							if(subtree_equal(prefit, par, -2, false)) {
-								 tr.replace_index(locs.first->second, ind2);
-								 fit=tr.erase(fit);
-								 expression_modified=true;
-								 replaced=true;
-								 break;
-								 }
-							++prefit;
-							}
-					  }
-				 if(replaced) break;
-				 }
-			if(replaced) break;
-			
-         // 2nd index to 1st index conversion?
-			locs=ind_dummy.equal_range(exptree(ind2));
-			while(locs.first!=locs.second) {
-				 if(locs.first->second!=(iterator)ind2) {
-					  // Does this index sit on an object in the "preferred form" list?
-					  iterator par=tr.parent(locs.first->second);
-					  sibling_iterator prefit=tr.begin(objs);
-					  while(prefit!=tr.end(objs)) {
-							if(subtree_equal(prefit, par, -2, false)) {
-								 tr.replace_index(locs.first->second, ind1);
-								 fit=tr.erase(fit);
-								 expression_modified=true;
-								 replaced=true;
-								 break;
-								 }
-							++prefit;
-							}
-					  }
-				 if(replaced) break;
-				 ++locs.first;
-				 }
-			 }
-		if(!replaced)
-			 ++fit;
-		}
-	if(expression_modified) {
-		cleanup_sums_products(tr, it);
-		return l_applied;
-		}
-	else return l_no_action;
-	}
-
 eliminate_metric::eliminate_metric(exptree& tr, iterator it)
+	: eliminate_converter(tr, it)
+	{
+	}
+
+bool eliminate_vielbein::is_conversion_object(iterator fit) const 
+	{
+	const Vielbein        *vb=properties::get<Vielbein>(fit);
+	const InverseVielbein *ivb=properties::get<InverseVielbein>(fit);
+	
+	if(vb || ivb)  return true;
+	else return false;
+	}
+
+bool eliminate_metric::is_conversion_object(iterator fit) const 
+	{
+	const Metric        *vb=properties::get<Metric>(fit);
+	const InverseMetric *ivb=properties::get<InverseMetric>(fit);
+	
+	if(vb || ivb)  return true;
+	else return false;
+	}
+
+eliminate_converter::eliminate_converter(exptree& tr, iterator it)
 	: algorithm(tr, it)
 	{
 	}
 
-void eliminate_metric::description() const
+void eliminate_converter::description() const
 	{
-	txtout << "Eliminate metrics." << std::endl;
 	}
 
-bool eliminate_metric::can_apply(iterator it)
+bool eliminate_converter::can_apply(iterator it)
 	{
 	if(*it->name=="\\prod") return true;
 	return false;
 	}
 
-algorithm::result_t eliminate_metric::apply(iterator& it)
+bool eliminate_converter::handle_one_index(iterator ind1, iterator ind2, iterator fit, sibling_iterator objs)
+	{
+	bool replaced=false;
+
+	std::pair<index_map_t::const_iterator,index_map_t::const_iterator> locs=
+		ind_dummy.equal_range(exptree(ind1));
+	if(std::distance(locs.first, locs.second)==2) {
+//				txtout << "index " << *ind1->name << std::endl;
+		while(locs.first!=locs.second) {
+			if(locs.first->second!=(iterator)ind1) {
+				// Does this index sit on an object in the "preferred form" list?
+				// (if there is no preferred form, always eliminate)
+				if(separated_by_derivative(locs.first->second, ind2, fit)==false) {
+					if(objs==args_end()) { // no
+						tr.move_ontop(locs.first->second, iterator(ind2))->fl.parent_rel=ind2->fl.parent_rel;;
+						fit=tr.erase(fit);
+						expression_modified=true;
+						replaced=true;
+						}
+					else { // yes
+						iterator par=tr.parent(locs.first->second);
+						sibling_iterator prefit=tr.begin(objs);
+						while(prefit!=tr.end(objs)) {
+							if(subtree_equal(prefit, par, -1, false)) {
+								tr.move_ontop(locs.first->second, iterator(ind2))->fl.parent_rel=ind2->fl.parent_rel;;
+								fit=tr.erase(fit);
+								expression_modified=true;
+								replaced=true;
+								break;
+								}
+							++prefit;
+							}
+						}
+					}
+				}
+			if(replaced) 
+				return true;
+			++locs.first;
+			}
+		}
+	return false;
+	}
+
+
+algorithm::result_t eliminate_converter::apply(iterator& it)
 	{
 	// Put arguments in canonical form.
 
 	sibling_iterator objs=args_begin();
-//	sibling_iterator vielb=objs;
-//	++vielb;
+
 	if(objs!=args_end())
 		if(*objs->name!="\\comma")
 			objs=tr.wrap(objs, str_node("\\comma"));
 
-	index_map_t ind_dummy, ind_free;
+	ind_free.clear();
+	ind_dummy.clear();
+
 	classify_indices(it, ind_free, ind_dummy);
 
-	// Run over all factors, find vielbeine, figure out whether they can
+	// Run over all factors, find metrics, figure out whether they can
 	// be used to turn the indices on the other tensors to preferred type.
 
 	sibling_iterator fit=tr.begin(it);
 	while(fit!=tr.end(it)) {
-		const Metric        *vb=properties::get<Metric>(fit);
-		const InverseMetric *ivb=properties::get<InverseMetric>(fit);
-
-		bool replaced=false;
-		if(vb || ivb) {
+		if(is_conversion_object(fit)) {
 			sibling_iterator ind1=tr.begin(fit), ind2=ind1;
 			++ind2;
-			
+
          // 1st index to 2nd index conversion?
-			std::pair<index_map_t::const_iterator,index_map_t::const_iterator> locs=
-				ind_dummy.equal_range(exptree(ind1));
-			if(std::distance(locs.first, locs.second)==2) {
-//				txtout << "index " << *ind1->name << std::endl;
-				while(locs.first!=locs.second) {
-					if(locs.first->second!=(iterator)ind1) {
-						// Does this index sit on an object in the "preferred form" list?
-						// (if there is no preferred form, always eliminate)
-						if(objs==args_end()) {
-							tr.move_ontop(locs.first->second, iterator(ind2))->fl.parent_rel=ind2->fl.parent_rel;;
-							fit=tr.erase(fit);
-//							exptree::print_recursive_treeform(txtout, it);
-							expression_modified=true;
-							replaced=true;
-							break;
-							}
-						else {
-							iterator par=tr.parent(locs.first->second);
-							sibling_iterator prefit=tr.begin(objs);
-							while(prefit!=tr.end(objs)) {
-								if(subtree_equal(prefit, par, -1, false)) {
-									tr.move_ontop(locs.first->second, iterator(ind2))->fl.parent_rel=ind2->fl.parent_rel;;
-									fit=tr.erase(fit);
-									expression_modified=true;
-									replaced=true;
-									break;
-									}
-								++prefit;
-								}
-							}
-						}
-					if(replaced) break;
-					++locs.first;
-					}
-				}
+			if(handle_one_index(ind1, ind2, fit, objs))
+				break;
 			
 			// 2nd index to 1st index conversion?
-			locs=ind_dummy.equal_range(exptree(ind2));
-			if(std::distance(locs.first, locs.second)==2) {
-				while(locs.first!=locs.second) {
-					if(!replaced && locs.first->second!=(iterator)ind2) {
-						// Does this index sit on an object in the "preferred form" list?
-//						txtout << "testing second index" << std::endl;
-						// (if there is no preferred form, always eliminate)
-						if(objs==args_end()) {
-							tr.move_ontop(locs.first->second, iterator(ind1))->fl.parent_rel=ind1->fl.parent_rel;
-							fit=tr.erase(fit);
-//							exptree::print_recursive_treeform(txtout, it);
-							expression_modified=true;
-							replaced=true;
-							break;
-							}
-						else {
-							iterator par=tr.parent(locs.first->second);
-							sibling_iterator prefit=tr.begin(objs);
-							while(prefit!=tr.end(objs)) {
-//							txtout << "testing " << *prefit->name << " against " << *par->name << std::endl;
-								if(subtree_equal(prefit, par, -1, false)) {
-//								txtout << "yes" << std::endl;
-									tr.move_ontop(locs.first->second, iterator(ind1))->fl.parent_rel=ind1->fl.parent_rel;
-									fit=tr.erase(fit);
-									expression_modified=true;
-									replaced=true;
-									break;
-									}
-								++prefit;
-								}
-							}
-						}
-					if(replaced) break;
-					++locs.first;
-					}
-				}
+			if(handle_one_index(ind2, ind1, fit, objs))
+				break;
 			}
-		if(!replaced)
-			++fit;
+		++fit;
 		}
-//	exptree::print_recursive_treeform(txtout, it);
+
 	if(expression_modified) {
 		cleanup_sums_products(tr, it);
 		return l_applied;
@@ -468,7 +371,7 @@ algorithm::result_t rewrite_indices::apply(iterator& it)
 
 				if(newtype->set_name == origtype->set_name) {
 //					txtout << "index already has same type" << std::endl;
-					if(origtype->position_free || walk->fl.parent_rel==dit->second->fl.parent_rel) {
+					if(origtype->position_type==Indices::free || walk->fl.parent_rel==dit->second->fl.parent_rel) {
 //						txtout << "and position is also already fine" << std::endl;
 						continue; // already fine
 						}
@@ -481,8 +384,8 @@ algorithm::result_t rewrite_indices::apply(iterator& it)
 
 				if(origtype->set_name == itype1->set_name && newtype->set_name == itype2->set_name) {
 //					txtout << "hit 1" << std::endl;
-					if( itype1->position_free || dit->second->fl.parent_rel == pr1 ) {
-						if( itype2->position_free || walk->fl.parent_rel != pr2 ) {
+					if( itype1->position_type==Indices::free || dit->second->fl.parent_rel == pr1 ) {
+						if( itype2->position_type==Indices::free || walk->fl.parent_rel != pr2 ) {
 //							txtout << "activate" << std::endl;
 							tr.replace_index(vbi1, dit->second);
 							exptree nd=get_dummy(itype2, par);
@@ -495,8 +398,8 @@ algorithm::result_t rewrite_indices::apply(iterator& it)
 					}
 				else if(origtype->set_name == itype2->set_name && newtype->set_name == itype1->set_name) {
 //					txtout << "hit 2" << std::endl;
-					if( itype2->position_free || dit->second->fl.parent_rel == pr2 ) {
-						if( itype1->position_free || walk->fl.parent_rel != pr1 ) {
+					if( itype2->position_type==Indices::free || dit->second->fl.parent_rel == pr2 ) {
+						if( itype1->position_type==Indices::free || walk->fl.parent_rel != pr1 ) {
 							tr.replace_index(vbi2, dit->second);
 							exptree nd=get_dummy(itype1, par);
 							tr.replace_index(vbi1,nd.begin());
