@@ -1,7 +1,7 @@
 /* 
 
 	Cadabra: a field-theory motivated computer algebra system.
-	Copyright (C) 2001-2009  Kasper Peeters <kasper.peeters@aei.mpg.de>
+	Copyright (C) 2001-2010  Kasper Peeters <kasper.peeters@aei.mpg.de>
 
    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -209,6 +209,7 @@ algorithm::result_t substitute::apply(iterator& st)
 	std::vector<iterator> subtree_insertion_points;
 	while(it!=repl.end()) { 
 		bool is_stripped=false;
+//		tr.print_recursive_treeform(std::cerr, repl.begin());
 
 //		For some reason 'a?' is not found!?! Well, that's presumably because _{a?} does not
 //      match ^{a?}. (though this does match when we write 'i' instead of a?. 
@@ -222,8 +223,9 @@ algorithm::result_t substitute::apply(iterator& st)
 			 }
 
 		if(loc!=comparator.replacement_map.end()) { // name wildcards
-//			std::cerr << "rule : " << *((*loc).first.begin()->name) << " -> " << std::endl;
-//			std::cerr << "going to replace " << *it->name << " with " << *((*loc).second.begin()->name) << std::endl;
+//		std::cerr << "rule : " << *((*loc).first.begin()->name) << " -> " 
+//					 << *((*loc).second.begin()->name) << std::endl;
+//		std::cerr << "going to replace " << *it->name << " with " << *((*loc).second.begin()->name) << std::endl;
 
 			// When a replacement is made here, and the index is actually
 			// a dummy in the replacement, we screw up the ind_dummy
@@ -236,8 +238,13 @@ algorithm::result_t substitute::apply(iterator& st)
 			ind_dummy.erase(exptree(it));
 
 			str_node::bracket_t remember_br=it->fl.bracket;
-			if(is_stripped)
-				 it->name=(*loc).second.begin()->name;
+			if(is_stripped || (it->is_name_wildcard() && !it->is_index()) ) { 
+            // a?_{i j k} type patterns should only replace the head
+				// TODO: should we replace brackets here too?
+				it->name=(*loc).second.begin()->name;
+				it->multiplier=(*loc).second.begin()->multiplier;
+				it->fl=(*loc).second.begin()->fl;
+				}
 			else
 				 it=tr.replace_index(it, (*loc).second.begin());
 			it->fl.bracket=remember_br;
@@ -264,6 +271,7 @@ algorithm::result_t substitute::apply(iterator& st)
 			}
 		else ++it;
 		}
+//	tr.print_recursive_treeform(std::cerr, repl.begin());
 
 	// If the replacement contains dummies, avoid clashes introduced when
 	// free indices in the replacement (induced from the original expression)
@@ -385,6 +393,30 @@ algorithm::result_t substitute::apply(iterator& st)
 	}
 
 
+/*  bug: cadabra-34.
+
+    Vary should take into account the depth of an object in a more clever way than
+	 is currently done. Consider an expression
+
+         A \partial{ A C + B } + D A;
+
+    and A->a, B->b etc. This should vary to
+
+         a  \partial{ A C + B } + A \partial{ a C + A c + b } + d A + a D;
+
+    Right now it produces a total mess for the partial derivative, because it does
+	 not understand that the depth counting for factors inside the partial involves
+	 knowing about the top-level product.
+
+    So what we should do is introduce a 'factor depth' and a 'sum index', which equal
+
+         A \partial{ A C + B } + D A;
+         1           1 1   1     1 1
+			1           1 1   1     2 2
+
+    For all 
+ */
+
 vary::vary(exptree& tr, iterator it)
 	: algorithm(tr, it)
 	{
@@ -398,13 +430,110 @@ void vary::description() const
 bool vary::can_apply(iterator it) 
 	{
 	if(*it->name=="\\prod") return true;
+	if(*it->name=="\\sum") return true;
 	if(is_single_term(it)) return true;
+	if(is_nonprod_factor_in_prod(it)) return true;
+	const Derivative *der = properties::get<Derivative>(it);
+	if(der) return true;
+	der = properties::get<Derivative>(tr.parent(it));
+	if(der) return true;
+	const Accent *acc = properties::get<Accent>(it);
+	if(acc) return true;
+	acc = properties::get<Accent>(tr.parent(it));
+	if(acc) return true;
 	return false;
 	}
 
 algorithm::result_t vary::apply(iterator& it)
 	{
-	if(is_single_term(it)) { // easy: just vary this term
+	const Derivative *der = properties::get<Derivative>(it);
+	const Accent     *acc = properties::get<Accent>(it);
+	if(der || acc) {
+		vary vry(tr, this_command);
+
+		sibling_iterator sib=tr.begin(it);
+		while(sib!=tr.end(it)) {
+			iterator app=sib;
+			++sib;
+			if(app->is_index()) continue;
+			if(vry.can_apply(app)) {
+				vry.apply(app);
+				}// not complete: should remove object when zero
+			}
+		expression_modified=true;
+		return l_applied;		
+		}
+
+	if(*it->name=="\\prod") {
+		exptree result;
+		result.set_head(str_node("\\expression"));
+		iterator newsum=result.append_child(result.begin(), str_node("\\sum"));
+		
+		// Iterate over all factors, attempting a substitute. If this
+		// succeeds, copy the term to the "result" tree. Then restore the
+		// original. We have to do the substitute on the original tree so
+		// that index relabelling takes into account the rest of the tree.
+		
+		exptree prodcopy(it); // keep a copy to restore after each substitute
+		
+		vary subs(tr, this_command);
+		int pos=0;
+		for(;;) { 
+			sibling_iterator fcit=tr.begin(it);
+			fcit+=pos;
+			if(fcit==tr.end(it)) break;
+			
+			iterator fcit2(fcit);
+			if(subs.can_apply(fcit2)) {
+				subs.apply(fcit2);
+				expression_modified=true;
+				
+				if(fcit2->is_zero()==false)
+					iterator newterm=result.append_child(newsum, it);
+
+				// restore original
+				it=tr.replace(it, prodcopy.begin());
+				}
+			++pos;
+			}
+		if(expression_modified && tr.number_of_children(newsum)>0) {
+			it=tr.move_ontop(it, newsum);
+			cleanup_nests(tr, it);
+			cleanup_expression(tr, it);
+			}
+		else { // varying any of the factors produces nothing, variation is zero
+			zero(it->multiplier);
+			expression_modified=true;
+			}
+		return l_applied;
+		}
+
+	if(*it->name=="\\sum") { // call vary on every term
+		vary vry(tr, this_command);
+
+		sibling_iterator sib=tr.begin(it);
+		while(sib!=tr.end(it)) {
+			iterator app=sib;
+			++sib;
+			if(vry.can_apply(app)) {
+				vry.apply(app);
+				if(app->is_zero()) {
+					tr.erase(app);
+					}
+				}
+			else {
+				// remove this term
+				tr.erase(app);
+				}
+			}
+		expression_modified=true;
+		return l_applied;
+		}
+	
+	der = properties::get<Derivative>(tr.parent(it));
+	acc = properties::get<Accent>(tr.parent(it));
+
+	if(der || acc || is_single_term(it)) { // easy: just vary this term by substitution
 		substitute subs(tr, this_command);
 		if(subs.can_apply(it)) {
 			if(subs.apply(it)==l_applied) {
@@ -414,45 +543,22 @@ algorithm::result_t vary::apply(iterator& it)
 			}
 		return l_no_action;
 		}
-
-	exptree result;
-	result.set_head(str_node("\\expression"));
-	iterator newsum=result.append_child(result.begin(), str_node("\\sum"));
-
-	// Iterate over all factors, attempting a substitute. If this
-	// succeeds, copy the term to the "result" tree. Then restore the
-	// original. We have to do the substitute on the original tree so
-	// that index relabelling takes into account the rest of the tree.
-
-	exptree prodcopy(it); // keep a copy to restore after each substitute
-
-	substitute subs(tr, this_command);
-	int pos=0;
-	for(;;) { 
-		sibling_iterator fcit=tr.begin(it);
-		fcit+=pos;
-		if(fcit==tr.end(it)) break;
-
-		iterator fcit2(fcit);
-		if(subs.apply_recursive(fcit2, false)) {
-			expression_modified=true;
-			iterator newterm=result.append_child(newsum, it);
-			// restore original
-			it=tr.replace(it, prodcopy.begin());
+	
+	if(is_nonprod_factor_in_prod(it)) {
+		substitute subs(tr, this_command);
+		if(subs.can_apply(it)) {
+			if(subs.apply(it)==l_applied) {
+				expression_modified=true;
+				return l_applied;
+				}
 			}
-		++pos;
-		}
-	if(expression_modified) {
-		it=tr.move_ontop(it, newsum);
-		cleanup_nests(tr, it);
-		cleanup_expression(tr, it);
-		}
-	else { // substitute didn't act anywhere, variation is zero
+		// else set to zero
 		zero(it->multiplier);
 		expression_modified=true;
+		return l_applied;
 		}
 
-	return l_applied;
+	return l_no_action;
 	}
 
 
