@@ -515,6 +515,27 @@ exptree::iterator exptree::move_index(iterator pos, const iterator& from)
 	return from;
 	}
 
+void exptree::list_wrap_single_element(iterator& it)
+	{
+	if(*it->name!="\\comma") {
+		iterator commanode=insert(it, str_node("\\comma"));
+		sibling_iterator fr=it, to=it;
+		++to;
+		reparent(commanode, fr, to);
+		it=commanode;
+		}
+	}
+
+void exptree::list_unwrap_single_element(iterator& it)
+	{
+	if(*it->name=="\\comma") {
+		if(number_of_children(it)==1) {
+			flatten(it);
+			it=erase(it);
+			}
+		}
+	}
+
 exptree::iterator exptree::flatten_and_erase(iterator pos)
 	{
 	multiplier_t tmp=*pos->multiplier;
@@ -535,6 +556,18 @@ unsigned int exptree::number_of_indices(iterator it)
 	return res;
 	}
 
+unsigned int exptree::number_of_direct_indices(iterator it) const
+	{
+	unsigned int res=0;
+	sibling_iterator sib=begin(it);
+	while(sib!=end(it)) {
+		if(sib->fl.parent_rel==str_node::p_sub || sib->fl.parent_rel==str_node::p_super)
+			++res;
+		++sib;
+		}
+	return res;
+	}
+
 exptree::index_iterator::index_iterator()
 	: iterator_base()
 	{
@@ -771,6 +804,15 @@ bool str_node::is_rational() const
 	return (*name=="1");
 	}
 
+bool str_node::is_integer() const
+	{
+	if(*name=="1") {
+		if(multiplier->get_den()==1)
+			return true;
+		}
+	return false;
+	}
+
 bool str_node::is_unsimplified_rational() const
 	{
 	if((*name).size()==0) return false;
@@ -781,7 +823,7 @@ bool str_node::is_unsimplified_rational() const
 	return true;
 	}
 
-bool str_node::is_integer() const
+bool str_node::is_unsimplified_integer() const
 	{
 	if((*name).size()==0) return false;
 	for(unsigned int i=0; i<(*name).size(); ++i) {
@@ -1006,7 +1048,7 @@ int subtree_compare(exptree::iterator one, exptree::iterator two,
 	int  mult=1;
 	if(one->is_index() && two->is_index() && one->is_rational() && two->is_rational()) mult=2;
 
-	bool position_free=true;
+	Indices::position_t position_type=Indices::free;
 	if(one->is_index() && two->is_index() && checksets) {
 		const Indices *ind1=properties::get<Indices>(one);
 		const Indices *ind2=properties::get<Indices>(two);
@@ -1021,11 +1063,11 @@ int subtree_compare(exptree::iterator one, exptree::iterator two,
 			}
 //		else std::cout << *one->name << " and " << *two->name << " in same set" << std::endl;
 		if(ind1!=0 && ind1==ind2) 
-			position_free=ind1->position_free;
+			position_type=ind1->position_type;
 		}
 	
 	// Compare sub/superscript relations.
-	if((mod_prel==-2 && !position_free) && one->is_index()) {
+	if((mod_prel==-2 && position_type!=Indices::free) && one->is_index()) {
 		if(one->fl.parent_rel!=two->fl.parent_rel) {
 			if(one->fl.parent_rel==str_node::p_sub) return 2;
 			else return -2;
@@ -1581,7 +1623,11 @@ int exptree_ordering::can_swap_prod_obj(exptree::iterator prod, exptree::iterato
 	while(sib!=prod.end()) {
 		const Indices *ind1=properties::get_composite<Indices>(sib);
 		const Indices *ind2=properties::get_composite<Indices>(obj);
-		if(! (ind1!=0 && ind2!=0) ) {
+		if(! (ind1!=0 && ind2!=0) ) { // If both objects are actually real indices, 
+			                           // then we do not include their commutativity property
+			                           // in the sign. This is because the routines that use
+                                    // can_swap_prod_obj all test for such index-index 
+                                    // swaps separately.
 			int es=subtree_compare(sib, obj);
 			sign*=can_swap(sib, obj, es, ignore_implicit_indices);
 			if(sign==0) break;
@@ -1746,13 +1792,16 @@ int exptree_ordering::can_swap(exptree::iterator one, exptree::iterator two, int
 		return sc1->sign();
 
 	// One or both of the objects are not in an explicit list. So now comes the generic
-	// part. The first step is to look at all indices of the two objects and determine 
+	// part. The first step is to look at all explicit indices of the two objects and determine 
 	// their commutativity. 
+	// Note: this does not yet look at arguments (non-index children).
 
 	int tmpsign=can_swap_ilist_ilist(one, two);
 	if(tmpsign==0) return 0;
 	
-	// The second step is to check for product-like and sum-like behaviour.
+	// The second step is to check for product-like and sum-like behaviour. The following
+	// take into account all commutativity properties of explict with implicit indices,
+	// as well as hard-specified commutativity of factors.
 
 	const CommutingAsProduct *comap1 = properties::get_composite<CommutingAsProduct>(one);
 	const CommutingAsProduct *comap2 = properties::get_composite<CommutingAsProduct>(two);
@@ -1822,8 +1871,8 @@ bool exptree_comparator::satisfies_conditions(exptree::iterator conditions, std:
 			exptree::sibling_iterator rhs=lhs;
 			++rhs;
 			properties::registered_property_map_t::iterator pit=
-				properties::registered_properties.find(*rhs->name);
-			if(pit==properties::registered_properties.end()) {
+				properties::registered_properties.store.find(*rhs->name);
+			if(pit==properties::registered_properties.store.end()) {
 				std::ostringstream str;
 				str << "Property \"" << *rhs->name << "\" not registered." << std::endl;
 				error=str.str();
